(function() {
  var Promise, SSL_RECORD_TYPES, Server, _, agent, allowDestroy, connect, debug, fs, getProxyForUrl, https, net, parse, ref, semaphore, sslSemaphores, sslServers, url,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require("lodash");

  ref = require("../../network"), agent = ref.agent, connect = ref.connect;

  allowDestroy = require("server-destroy-vvo");

  debug = require("debug")("cypress:https-proxy");

  fs = require("fs-extra");

  getProxyForUrl = require("proxy-from-env").getProxyForUrl;

  https = require("https");

  net = require("net");

  parse = require("./util/parse");

  Promise = require("bluebird");

  semaphore = require("semaphore");

  url = require("url");

  fs = Promise.promisifyAll(fs);

  sslServers = {};

  sslSemaphores = {};

  SSL_RECORD_TYPES = [22, 128, 0];

  Server = (function() {
    function Server(_ca, _port) {
      this._ca = _ca;
      this._port = _port;
      this._onError = null;
    }

    Server.prototype.connect = function(req, browserSocket, head, options) {
      if (options == null) {
        options = {};
      }
      browserSocket.setNoDelay(true);
      debug("Writing browserSocket connection headers %o", {
        url: req.url,
        headLength: _.get(head, 'length'),
        headers: req.headers
      });
      browserSocket.on("error", (function(_this) {
        return function(err) {
          return debug("received error on client browserSocket %o", {
            err: err,
            url: req.url
          });
        };
      })(this));
      browserSocket.write("HTTP/1.1 200 OK\r\n");
      if (req.headers["proxy-connection"] === "keep-alive") {
        browserSocket.write("Proxy-Connection: keep-alive\r\n");
        browserSocket.write("Connection: keep-alive\r\n");
      }
      browserSocket.write("\r\n");
      if (_.get(head, "length")) {
        return this._onFirstHeadBytes(req, browserSocket, head, options);
      }
      return browserSocket.once("data", (function(_this) {
        return function(data) {
          return _this._onFirstHeadBytes(req, browserSocket, data, options);
        };
      })(this));
    };

    Server.prototype._onFirstHeadBytes = function(req, browserSocket, head, options) {
      var odc;
      debug("Got first head bytes %o", {
        url: req.url,
        head: _.chain(head).invoke('toString').slice(0, 64).join('').value()
      });
      browserSocket.pause();
      if (odc = options.onDirectConnection) {
        if (odc.call(this, req, browserSocket, head) === true) {
          return this._makeDirectConnection(req, browserSocket, head);
        } else {
          debug("Not making direct connection %o", {
            url: req.url
          });
        }
      }
      return this._onServerConnectData(req, browserSocket, head);
    };

    Server.prototype._onUpgrade = function(fn, req, browserSocket, head) {
      if (fn) {
        return fn.call(this, req, browserSocket, head);
      }
    };

    Server.prototype._onRequest = function(fn, req, res) {
      var hostPort;
      hostPort = parse.hostAndPort(req.url, req.headers, 443);
      req.url = url.format({
        protocol: "https:",
        hostname: hostPort.host,
        port: hostPort.port
      }) + req.url;
      if (fn) {
        return fn.call(this, req, res);
      }
      return req.pipe(request(req.url)).on("error", function() {
        res.statusCode = 500;
        return res.end();
      }).pipe(res);
    };

    Server.prototype._getProxyForUrl = function(urlStr) {
      var port;
      port = Number(_.get(url.parse(urlStr), 'port'));
      debug('getting proxy URL %o', {
        port: port,
        serverPort: this._port,
        sniPort: this._sniPort,
        url: urlStr
      });
      if ([this._sniPort, this._port].includes(port)) {
        return void 0;
      }
      return getProxyForUrl(urlStr);
    };

    Server.prototype._makeDirectConnection = function(req, browserSocket, head) {
      var hostname, port, ref1;
      ref1 = url.parse("https://" + req.url), port = ref1.port, hostname = ref1.hostname;
      debug("Making connection to " + hostname + ":" + port);
      return this._makeConnection(browserSocket, head, port, hostname);
    };

    Server.prototype._makeConnection = function(browserSocket, head, port, hostname) {
      var onSocket, upstreamProxy;
      onSocket = (function(_this) {
        return function(err, upstreamSocket) {
          var onError;
          debug('received upstreamSocket callback for request %o', {
            port: port,
            hostname: hostname,
            err: err
          });
          onError = function(err) {
            browserSocket.destroy(err);
            if (_this._onError) {
              return _this._onError(err, browserSocket, head, port);
            }
          };
          if (err) {
            return onError(err);
          }
          upstreamSocket.setNoDelay(true);
          upstreamSocket.on("error", onError);
          browserSocket.pipe(upstreamSocket);
          upstreamSocket.pipe(browserSocket);
          upstreamSocket.write(head);
          return browserSocket.resume();
        };
      })(this);
      if (upstreamProxy = this._getProxyForUrl("https://" + hostname + ":" + port)) {
        debug("making proxied connection %o", {
          host: hostname + ":" + port,
          proxy: upstreamProxy
        });
        return agent.httpsAgent.createUpstreamProxyConnection({
          proxy: upstreamProxy,
          href: "https://" + hostname + ":" + port,
          uri: {
            port: port,
            hostname: hostname
          },
          shouldRetry: true
        }, onSocket);
      }
      return connect.createRetryingSocket({
        port: port,
        host: hostname
      }, onSocket);
    };

    Server.prototype._onServerConnectData = function(req, browserSocket, head) {
      var firstBytes, hostname, makeConnection, sem, sslServer;
      firstBytes = head[0];
      makeConnection = (function(_this) {
        return function(port) {
          debug("Making intercepted connection to %s", port);
          return _this._makeConnection(browserSocket, head, port, "localhost");
        };
      })(this);
      if (indexOf.call(SSL_RECORD_TYPES, firstBytes) < 0) {
        return makeConnection(this._port);
      }
      hostname = url.parse("https://" + req.url).hostname;
      if (sslServer = sslServers[hostname]) {
        return makeConnection(sslServer.port);
      }
      if (!(sem = sslSemaphores[hostname])) {
        sem = sslSemaphores[hostname] = semaphore(1);
      }
      return sem.take((function(_this) {
        return function() {
          var leave;
          leave = function() {
            return process.nextTick(function() {
              return sem.leave();
            });
          };
          if (sslServer = sslServers[hostname]) {
            leave();
            return makeConnection(sslServer.port);
          }
          return _this._getPortFor(hostname).then(function(port) {
            sslServers[hostname] = {
              port: port
            };
            leave();
            return makeConnection(port);
          });
        };
      })(this));
    };

    Server.prototype._normalizeKeyAndCert = function(certPem, privateKeyPem) {
      return {
        key: privateKeyPem,
        cert: certPem
      };
    };

    Server.prototype._getCertificatePathsFor = function(hostname) {
      return this._ca.getCertificateKeysForHostname(hostname).spread(this._normalizeKeyAndCert);
    };

    Server.prototype._generateMissingCertificates = function(hostname) {
      return this._ca.generateServerCertificateKeys(hostname).spread(this._normalizeKeyAndCert);
    };

    Server.prototype._getPortFor = function(hostname) {
      return this._getCertificatePathsFor(hostname)["catch"]((function(_this) {
        return function(err) {
          return _this._generateMissingCertificates(hostname);
        };
      })(this)).then((function(_this) {
        return function(data) {
          if (data == null) {
            data = {};
          }
          _this._sniServer.addContext(hostname, data);
          return _this._sniPort;
        };
      })(this));
    };

    Server.prototype.listen = function(options) {
      if (options == null) {
        options = {};
      }
      return new Promise((function(_this) {
        return function(resolve) {
          _this._onError = options.onError;
          _this._sniServer = https.createServer({});
          allowDestroy(_this._sniServer);
          _this._sniServer.on("upgrade", _this._onUpgrade.bind(_this, options.onUpgrade));
          _this._sniServer.on("request", _this._onRequest.bind(_this, options.onRequest));
          return _this._sniServer.listen(0, '127.0.0.1', function() {
            _this._sniPort = _this._sniServer.address().port;
            debug("Created SNI HTTPS Proxy on port %s", _this._sniPort);
            return resolve();
          });
        };
      })(this));
    };

    Server.prototype.close = function() {
      var close;
      close = (function(_this) {
        return function() {
          return new Promise(function(resolve) {
            return _this._sniServer.destroy(resolve);
          });
        };
      })(this);
      return close()["finally"](function() {
        return sslServers = {};
      });
    };

    return Server;

  })();

  module.exports = {
    reset: function() {
      return sslServers = {};
    },
    create: function(ca, port, options) {
      var srv;
      if (options == null) {
        options = {};
      }
      srv = new Server(ca, port);
      return srv.listen(options)["return"](srv);
    }
  };

}).call(this);
