/* global Blob File */

/*
 * Module requirements.
 */

let isArray = require('isarray')

let toString = Object.prototype.toString
let withNativeBlob = typeof global.Blob === 'function' || toString.call(global.Blob) === '[object BlobConstructor]'
let withNativeFile = typeof global.File === 'function' || toString.call(global.File) === '[object FileConstructor]'

/**
 * Module exports.
 */

module.exports = function hasBinaryCircular (obj) {
  return hasBinary(obj, [])
}

/**
 * Checks for binary data.
 *
 * Supports Buffer, ArrayBuffer, Blob and File.
 *
 * @param {Object} anything
 * @api public
 */

function hasBinary (obj, known) {
  if (!obj || typeof obj !== 'object') {
    return false
  }

  if (known.indexOf(obj) >= 0) {
    return false
  }

  known.push(obj)

  if (isArray(obj)) {
    for (let i = 0, l = obj.length; i < l; i++) {
      if (hasBinary(obj[i], known)) {
        return true
      }
    }

    return false
  }

  if ((typeof global.Buffer === 'function' && global.Buffer.isBuffer && global.Buffer.isBuffer(obj)) ||
     (typeof global.ArrayBuffer === 'function' && obj instanceof ArrayBuffer) ||
     (withNativeBlob && obj instanceof Blob) ||
     (withNativeFile && obj instanceof File)
  ) {
    return true
  }

  // see: https://github.com/Automattic/has-binary/pull/4
  if (obj.toJSON && typeof obj.toJSON === 'function') {
    return hasBinary(obj.toJSON(), known)
  }

  for (let key in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, key) && hasBinary(obj[key], known)) {
      return true
    }
  }

  return false
}
